"use strict";

require("core-js/modules/es.object.define-property");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TablePlugin = TablePlugin;

var _tools = require("../tools/tools");

/**
 * [Table扩展语法]{ https://www.markdownguide.org/extended-syntax/#tables}
 * @param addToken
 * @constructor
 */
function TablePlugin() {}

TablePlugin.prototype.plugin = function (addToken, EL_TYPE) {
  addToken('td', true, {
    filterRule: {
      children: [EL_TYPE['all_element']]
    },
    convertRule: function convertRule(node) {
      return ' ';
    },
    endRule: function endRule(node) {
      return " |";
    }
  });
  addToken('th', true, {
    filterRule: {
      attribute: [{
        name: 'style'
      }],
      children: [EL_TYPE['all_element']]
    },
    convertRule: function convertRule(node) {
      return ' ';
    },
    endRule: function endRule(node) {
      return " |";
    }
  });
  addToken('tr', true, {
    filterRule: {
      children: [EL_TYPE['all_element']]
    },
    convertRule: function convertRule(node) {
      return '|';
    },
    endRule: function endRule(node) {
      return '\n';
    }
  });
  addToken('table', true, {
    filterRule: {
      children: [EL_TYPE['all_element']]
    },
    convertRule: function convertRule(node) {
      return '';
    },
    endRule: function endRule(node) {
      return '\n';
    }
  });
  addToken('thead', true, {
    filterRule: {
      children: [EL_TYPE['all_element']]
    },
    convertRule: function convertRule(node) {
      return '';
    },
    endRule: function endRule(node) {
      var ths = node.children[0].children;
      var str = '|',
          hyphens = '----',
          colon = ':',
          ws = ' ';

      for (var i = 0, len = ths.length, th, cssObj; i < len; i++) {
        th = ths[i];
        cssObj = _tools.Tools.formatCSS(th.attribute.style);

        switch (cssObj['text-align']) {
          case 'left':
            str += ws + colon + hyphens + ws;
            break;

          case 'center':
            str += ws + colon + hyphens + colon + ws;
            break;

          case 'right':
            str += ws + hyphens + colon + ws;
            break;

          default:
            str += ws + hyphens + ws;
        }

        str += '|';
      }

      return "".concat(str, "\n");
    }
  });
  addToken('tbody', true, {
    filterRule: {
      children: [EL_TYPE['all_element']]
    },
    convertRule: function convertRule(node) {
      return '';
    },
    endRule: function endRule(node) {
      return '';
    }
  });
};